<!--- 
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is comprised of the PT Tags directory

The Initial Developer of the Original Code is
PaperThin, Inc. Copyright(C) 2015.
All Rights Reserved.

By downloading, modifying, distributing, using and/or accessing any files 
in this directory, you agree to the terms and conditions of the applicable 
end user license agreement.
--->
<!---
/* *************************************************************** */
Author: 	
	PaperThin Inc.
Name:
	blog_tags_render.cfc
Description:
	Blog Tags CFT
	Customized from the PT Tags field to store the plain text for tags 
		and convert into the UUIDs during the blog post saving process. 
ADF App:
	PT Tags
Version:
	3.0.2
History:
	2010-03-27 - RLW - Created
	2011-10-28 - MFC - Customized for integration with PT Blog App.
	2012-03-30 - MFC - Fixed case issue with the JQuery "autocomplete".
	2012-09-11 - MFC - Commented out the "Show All" button.
	2013-02-12 - DMB - Commented out line 204 to fix error with tags.
	2013-02-25 - MFC - Commented out the "setTags" JS function b/c it is not needed.
	2015-05-14 - DJM - Converted to CFC
	2015-09-11 - GAC - Added a isMultiline() call so the label renders at the top
	2015-09-23 - GAC - Added logic to check to makes sure tags is enabled in the Blog App and that PT Tags is installed
	2016-02-26 - DRM - Resource detection support
--->
<cfcomponent displayName="BlogTags Render" extends="ADF.extensions.customfields.adf-form-field-renderer-base">

<!--- 
	// IMPORTATNT:  
		If the Tags Feature is disabled  this renderControl() will NOT be called. 
		See the renderStandard() OVERRIDE method below which renders the field 
		will be rendered as a hidden field.
		Although, if the Tags App is not installed but the Tag Feature is enabled the 
		renderControl() will be called and display a message on the form along with 
		a hidden field.
--->
<cffunction name="renderControl" returntype="void" access="public">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = Duplicate(arguments.parameters);
		var currentValue = arguments.value;	// the field's current value		
		var tagsField = "intellisense_#arguments.fieldName#";
		var blogID = 0;
		var tagsAppCheck = false;
		
		if ( StructKeyExists(request.params, "blogID") )  
			blogID = request.params.blogID;
		
		if ( StructKeyExists(application,"ptTags") )
			tagsAppCheck =  true;
		
		if ( tagsAppCheck ) 
		{
			application.ptTags.scripts.loadJQuery();
			application.ptTags.scripts.loadJQueryUI();
			//application.ptTags.scripts.loadJQueryAutocomplete();
		}
	</cfscript>
	
	<cfif tagsAppCheck>
		<cfoutput>
			<!--- Load in the styles --->
			<link rel="stylesheet" type="text/css" href="/ADF/apps/pt_tags/style/ptTags.css">
		</cfoutput>
		<cfscript>	
			renderJSFunctions(argumentCollection=arguments,blogID=blogID);
		</cfscript>
		<cfoutput>	
			<input type="text" name="#tagsField#" id="#tagsField#"><input type="button" name="#tagsField#_add" id="#tagsField#_add" value="Add to List">
			<!--- <input type="button" name="#tagsField#_btn" id="#tagsField#_btn" style="display:none" value="show all"> --->
			<div id="#tagsField#_tagBlock" class="ptTags_block">
				<ul id="#tagsField#_tags"></ul>
			</div>
			<input type="hidden" name="#arguments.fieldName#" id="#arguments.fieldName#" value="#currentValue#">
		</cfoutput>
	<cfelse>
		<cfoutput>	
			<div class="cs_dlgLabelSmall">The Tags field requires the PT Tags App to be installed.<br>
				Or the Tags feature can be disabled in the PT Blog App configuration.</div>
			<input type="hidden" name="#arguments.fieldName#" id="#arguments.fieldName#" value="#currentValue#">
		</cfoutput>
	</cfif>
</cffunction>

<cffunction name="renderJSFunctions" returntype="void" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	<cfargument name="blogID" type="string" required="yes">
	
	<cfscript>
		var currentValue = arguments.value;
		var tagsField = "intellisense_#arguments.fieldName#";
	</cfscript>
<cfoutput>
<script type="text/javascript">
<!--
var contentID = "";
var parentContentID = "";
var availTags = new Array();
	
// handle the submission of the tags field
function #arguments.fieldName#handleSave(){
	var i = 0;
	var tagArray = jQuery("###tagsField#_tags").sortable('toArray');
	var tagList = "";
	// serialize the tag array into a comma separated list of tags (to be sent to add ajax call)
	for( i; i < tagArray.length; i++ )
	{
		if( i != 0 )
			tagList = tagList + "^" + tagArray[i];
		else
			tagList = tagArray[i] ;
	}
	
	// send the list to be added and get back id's to store
	jQuery.ajax({
		url: "#application.ADF.ajaxProxy#",
		data: ({ bean: "tagsService", method: "handleTagAdd", tags: tagList, contentID: "#arguments.blogID#", parentContentID: parentContentID}),
		async: false,
		success: function(results){
			jQuery("###arguments.fieldName#").attr("value", results);
		}
	});
	return true;
}	
	
jQuery("document").ready( function() {
	// the contentID and parentContentID should be defined in this form somewhere
	//contentID = jQuery("##contentID").attr("value");
	//parentContentID = jQuery("##parentContentID").attr("value");
	//if( contentID.length == 0 )
		contentID = 0;
	//if( parentContentID.length == 0 )
		parentContentID = 0;
	
	<cfif len(currentValue)>
	// get the data for the current selected tags
	jQuery.ajax({
		url: "#application.ADF.ajaxProxy#",
		data: ({ bean: "tagsDAO", method: "getByTagIDList", tagIDList: "#currentValue#", returnFormat: "json" } ),
		async: false,
		dataType: "json",
		type: "post",
		success: function(results){
			// build the "tags" window
			for(var i=0; i < results.length; i++ ){
				if( results[i].VALUES.tag.length > 0 )
					#arguments.fieldName#addToList(results[i].VALUES.tag);
			}
		}	
	})
	</cfif>	
	
	// get the list of available tags
	jQuery('###tagsField#').bind('focus', #arguments.fieldName#getTags);
	// add onclick action to add
	jQuery("###tagsField#_add").click(function(){ #arguments.fieldName#addToList( jQuery("###tagsField#").val() ) });
	// make keywords sortable
	jQuery("###tagsField#_tags").sortable( { connectWith: "###tagsField#_tags", 
												scroll: true,
												update: function(event, ui) { 
													//setTags(); 
												} 
											} );
			
	//setTags();		
});

// adds tag to the list of tags
function #arguments.fieldName#addToList(tag){
	if( tag.length > 0  )
	{
		//TODO: Add some code that prevents tags with more than 3 spaces
		itemID = tag.replace(/'/gi, "&##39;");
		var item = "<li id='" + itemID  + "'><span>" + tag + "</span><div>x</div></li>";
		jQuery("###tagsField#_tags").append(item);
		jQuery("###tagsField#").val("");
		jQuery("###tagsField#_tags li div").click(#arguments.fieldName#doRemove);
		jQuery("###tagsField#").focus();
		
		//setTags();
	}
}
// removes tags from the list of tags
function #arguments.fieldName#doRemove(){
	if( confirm("Are you sure you want to remove '" + jQuery(this).parent().attr("id") + "' from the list" ) )
		jQuery(this).parent().remove();
	//setTags();
}
// retrieve the list of available tags
function #arguments.fieldName#getTags(){
	jQuery.ajax( {
		url: "#application.ADF.ajaxProxy#",
		data: ({ bean: "tagsService", method: "tagArrayByContentID", contentID: "#arguments.blogID#", returnFormat: "json" }),
		async: false,
		type: "post",
		dataType: "json",
		success: function(availTags){
			//alert(availTags);
			// convert json to array
			eval("'(' + availTags + ')'");
			//alert(availTags);
			if( availTags.length > 0 )
			{
				// bind the autocomplete to the returned array
				jQuery('###tagsField#').autocomplete({source: availTags});
				
				// bind click of "show all" to show data
				jQuery('###tagsField#_btn').autocomplete('button-supply');
				//jQuery("###tagsField#_btn").show();
			}
		},
		failure: function(results){
			alert("Failed to retrieve avilable tags. Click Ok to continue to add tags manually.");
		}
	});
}

// Sets the tag text into the list to store
/* function setTags(){
	// var tagArray = jQuery("###tagsField#_tags").sortable('toArray');
	// DMB - commented out the above line and added the following line to set tagArray equal to null.
	var tagArray = "";
	//console.log(tagArray);
	var tagList = "";
	var i=0;
					
	// serialize the tag array into a comma separated list of tags (to be sent to add ajax call)
	for( i; i < tagArray.length; i++ )
	{
		if( i != 0 )
			tagList = tagList + "^" + tagArray[i];
		else
			tagList = tagArray[i] ;
	}
	// Append the list as plain text into the field 
	jQuery("###arguments.fieldName#").attr("value", tagList);
} */
//-->
</script>
</cfoutput>
</cffunction>

<cfscript>
	public void function renderStandard()
	{
		var renderFullCFT = false;
		
		// If Blog Tags has been disabled or the tagsIntegrated has not been set
		// but the field has been added do to the Posts2 element then only render a hidden field.
		if ( StructKeyExists(application,"ptBlog2") 
				AND StructKeyExists(application.ptBlog2,"appConfig") 
				AND StructKeyExists(application.ptBlog2.appConfig,"tagsIntegrated")
				AND IsBoolean(application.ptBlog2.appConfig.tagsIntegrated) )
		{
			renderFullCFT = application.ptBlog2.appConfig.tagsIntegrated;
		}
		
		if ( !renderFullCFT )
			arguments.displayMode = "hidden";
			
		super.renderStandard(argumentCollection=arguments);
	}
	private any function getValidationJS(required string formName, required string fieldName, required boolean isRequired)
	{
		return '#arguments.fieldName#handleSave()';
	}
	private boolean function isMultiline()
	{
		return true;
	}

	public string function getResourceDependencies()
	{
		return listAppend(super.getResourceDependencies(), "jQuery,jQueryUI");
	}
	public string function loadResourceDependencies()
	{
		// actual loading done conditionally in renderControl()
	}
</cfscript>

</cfcomponent>